/*
 * TreeC.h - Simple random-generator.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 02-Aug-1998  
 *
 */

#ifndef TreeC_h
#define TreeC_h

#include <Error/LibraryErrorC.h>
#include <Error/InternalErrorC.h>

#include <vector>
#include <stack>
#include <iostream>

template <class T>
class TreeC
{
	public:

		// -----------------------------  Iterators

		class IteratorA
		{
			public:

				IteratorA
				(
					TreeC * iTree
				): Current(iTree)
				{
				};

				virtual ~IteratorA(){};

				virtual IteratorA & operator++() =0;
				virtual T & CurrentItem() const =0;
				virtual T & operator*() const =0;
				virtual T * operator->() const =0;

			protected:

				TreeC * Current;

			private:

		}; // IteratorA

		class ConstIteratorA
		{
			public:

				ConstIteratorA
				(
					TreeC const & iTree
				): Tree(iTree)
				{
				};

				virtual ~ConstIteratorA() =0;

				virtual void First();
				virtual void Next();
				virtual TreeC const & CurrentItem() const;

			protected:

				TreeC const & Tree;

			private:

		}; // ConstIteratorA

		class ConstIterator_PreorderC: public ConstIteratorA
		{
			public:

				ConstIterator_PreorderC
				(
					TreeC const & iTree
				): ConstIteratorA(iTree),Current(iTree)
				{};

	 			virtual ~ConstIterator_PreorderC();

				virtual void First();
				virtual void Next();
				virtual TreeC const & CurrentItem() const;

			protected:
				
				TreeC const & Current;

			private:

		}; // Iterator_PreorderC


		class Iterator_PreorderC: public IteratorA
		{
			public:

				Iterator_PreorderC
				(
					TreeC * iTree
				): IteratorA(iTree)
				{};

	 			virtual ~Iterator_PreorderC(){};

				virtual IteratorA & operator++();
				virtual IteratorA & operator++(int iInt)
				{
					if (iInt == 0)
						iInt = 1;
					for (int i = 0; i < iInt; ++i)
					{
						 ++(*this);
					}
					return *this;
				};

				virtual T* operator->() const
				{
					return &Current->Element;
				};

				virtual T & operator*() const
				{
					return Current->Element;
				};
				virtual T & CurrentItem() const
				{
					return Current->Element;
				};

			protected:

				stack<TreeC *> StillToDo;

			private:

		}; // Iterator_PreorderC

		class Iterator_PostorderC: public IteratorA
		{
			public:

				Iterator_PostorderC
				(
					TreeC * iTree
				): IteratorA(iTree)
				{
					while (Current->Child != 0)
					{
						StillToDo.push(Current);
						Current = Current->Child;
					}

				};

	 			virtual ~Iterator_PostorderC(){};

				virtual bool IsEnd() const
				{
					 return (Current == 0);
				};

				virtual IteratorA & operator++();
				virtual IteratorA & operator++(int iInt)
				{
					if (iInt == 0)
						iInt = 1;
					for (int i = 0; i < iInt; ++i)
					{
						 ++(*this);
						 if (this->Current == 0)
							 return 0;
					}

					return *this;
				};

				virtual T* operator->() const
				{
					return &Current->Element;
				};

				virtual T & operator*() const
				{
					return Current->Element;
				};
				virtual T & CurrentItem() const
				{
					return Current->Element;
				};

			protected:

				stack<TreeC *> StillToDo;

			private:

		}; // Iterator_PostorderC

		class Insert_IteratorC: public IteratorA
		{
			public:

				Insert_IteratorC
				(
					TreeC * iTree,
					IteratorA & iIterator
				): IteratorA(iTree), Iterator(iIterator)
				{};

	 			virtual ~Insert_IteratorC(){};

				virtual T* operator->() const
				{
					return &Current->Element;
				};

				virtual T & operator*() const
				{
					return Current->Element;
				};
				virtual T & CurrentItem() const
				{
					return Current->Element;
				};

			protected:

				IteratorA & Iterator;

			private:

		}; // Iterator_PreorderC

		// ----------------------------  Class
		
		TreeC
		(
			T & iElement
		): Element(iElement)
		{
			Parent = 0;
			Child = 0;
			Left = 0;
			Right = 0;
		};
		
		TreeC
		(
			TreeC * iParent,
			T & iElement
		): Parent(iParent), Element(iElement)
		{
			Right = 0;
			if (Parent->Child == 0)
			{
				 Parent->Child = this;
				 Left = 0;
			}
			else
			{
				 TreeC * tmp = Parent->Child;
				 while (tmp->Right != 0)
					 tmp = tmp->Right;
				 tmp->Right = this;
				 this->Left = tmp;
					 
			}
		};
		
		virtual ~TreeC()
		{
		};

		Iterator_PreorderC CreateIterator_Preorder()
		{
			 return Iterator_PreorderC(this);
		}
		friend TreeC<T>::Iterator_PreorderC;

		Iterator_PostorderC CreateIterator_Postorder()
		{
			 return Iterator_PostorderC(this);
		}
		friend TreeC<T>::Iterator_PostorderC;

	protected:

		/// This container element's left and right brother
		TreeC * Right;
		TreeC * Left;

		/// Its parent and first child
		TreeC * Parent;
		TreeC * Child;

		T & Element;

	private:

}; // TreeC

#endif


// Eof TreeC.h
