/*
 * GP_NodeA.h - An abstract node for the GP stuff.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 10-Aug-1998
 *
 */

#ifndef GP_NodeA_h
#define GP_NodeA_h

#include "RowC.h"
#include <Common/TreeC.h>

template <class T>
class GP_NodeA
{
	public:

		GP_NodeA(){};

		virtual ~GP_NodeA(){};

		virtual T const & Evaluate
		(
			TreeC<GP_NodeA<T> >::TreeElementC const &,
			RowC<T> const & iRow
		) =0;

		virtual unsigned int Arity() =0;

		friend ostream const & operator<<
		(
			ostream & iOstream,
			GP_NodeA const & iGP_Node
		)
		{
			 iGP_Node.Print(iOstream);

			 return iOstream;
		};

	protected:

		virtual void Print
		(
			ostream & iOstream
		) const =0;

	private:
	 
};

template <class T>
class TerminalA: public GP_NodeA<T>
{
	public:

		virtual ~TerminalA(){};
};

template <class T>
class FunctionA: public GP_NodeA<T>
{
	public:

	protected:

	private:
};

template <class T>
class VariableC: public TerminalA<T>
{
	public:

		VariableC
		(
			unsigned int iInt
		): VariableNumber(iInt)
		{};

		virtual ~VariableC(){};

		virtual T const & Evaluate
		(
			TreeC<GP_NodeA<T> >::TreeElementC const &,
			RowC<T> const & iRow
		)
		{
			return iRow.GetInput(VariableNumber);
		};

		virtual unsigned int Arity()
		{
			 return 0;
		};

	protected:

		unsigned int VariableNumber;

		virtual void Print
		(
			ostream & iOstream
		) const
		{
			 iOstream << "V" << VariableNumber;
		};

	private:

};

template <class T>
class ConstantC: public TerminalA<T>
{
	public:

		ConstantC
		(
			T const iT
		): TheConstant(iT)
		{};

		virtual ~ConstantC(){};

		virtual T const & Evaluate
		(
			TreeC<GP_NodeA<T> >::TreeElementC const &,
			RowC<T> const &
		)
		{
			return TheConstant;
		};

		virtual unsigned int Arity()
		{
			 return 0;
		};

	protected:

		virtual void Print
		(
			ostream & iOstream
		) const
		{
			 iOstream << TheConstant;
		};


		T const TheConstant;

	private:

};

class BooleanFunctionC: public FunctionA<bool>
{
};


class BooleanFunction_TWO_AND_C: public BooleanFunctionC
{
	 public:
		 
		virtual unsigned int Arity()
		{
			 return 2;
		};

		virtual bool const & Evaluate
		(
			TreeC<GP_NodeA<bool> >::TreeElementC const & iElement,
			RowC<bool> const & iRow
		)
		{
			/*
			cerr << "and";
			TreeC<GP_NodeA<bool> >::TreeElementC * i = iElement.Child;
			bool b = i->GetElement().Evaluate(*i, iRow);
			i = i->Right;
			b &= i->GetElement().Evaluate(*i, iRow);
			return b;
			*/
			TreeC<GP_NodeA<bool> >::TreeElementC::ChildConstIteratorC i = iElement.CreateChildConstIterator();
			bool b = i->Evaluate(i.GetTreeElement(), iRow);
			i++;
			b &= i->Evaluate(i.GetTreeElement(), iRow);
			return b;
			//return ((*iChildren).Evaluate(iChildren, iRow) && (*(iChildren++)).Evaluate(iChildren, iRow));
		};
		
	protected:
		
		virtual void Print
		(
			ostream & iOstream
		) const
		{
			 iOstream << "AND";
		};

};



class BooleanFunction_TWO_OR_C: public BooleanFunctionC
{
	 public:
		 
		virtual unsigned int Arity()
		{
			 return 2;
		};

		virtual bool const & Evaluate
		(
			TreeC<GP_NodeA<bool> >::TreeElementC const & iElement,
			RowC<bool> const & iRow
		)
		{
			/*
			cerr << "or";
			TreeC<GP_NodeA<bool> >::TreeElementC * i = iElement.Child;
			bool b = i->GetElement().Evaluate(*i, iRow);
			i = i->Right;
			b |= i->GetElement().Evaluate(*i, iRow);
			return b;
			*/

			TreeC<GP_NodeA<bool> >::TreeElementC::ChildConstIteratorC i = iElement.CreateChildConstIterator();
			bool b = i->Evaluate(i.GetTreeElement(), iRow);
			i++;
			b |= i->Evaluate(i.GetTreeElement(), iRow);
			return b;
			//return ((*iChildren).Evaluate(iChildren, iRow) || (*(iChildren++)).Evaluate(iChildren, iRow));
		};

	 protected:

		virtual void Print
		(
			ostream & iOstream
		) const
		{
			 iOstream << "OR";
		};

};



class BooleanFunction_ONE_NOT_C: public BooleanFunctionC
{
	 public:
		 
		virtual unsigned int Arity()
		{
			 return 1;
		};

		virtual bool const & Evaluate
		(
			TreeC<GP_NodeA<bool> >::TreeElementC const & iElement,
			RowC<bool> const & iRow
		)
		{
			/*
			cerr << "not";
			TreeC<GP_NodeA<bool> >::TreeElementC * i = iElement.Child;
			bool b;
			b = !( i->GetElement().Evaluate(*i, iRow) );
			return b;
			*/
			TreeC<GP_NodeA<bool> >::TreeElementC::ChildConstIteratorC i = iElement.CreateChildConstIterator();
			bool b = i->Evaluate(i.GetTreeElement(), iRow);
			//return (!(*iChildren).Evaluate(iChildren, iRow));
			return b;
		};

	 protected:

		virtual void Print
		(
			ostream & iOstream
		) const
		{
			 iOstream << "NOT";
		};

};

#endif


// Eof GP_NodeA.h

