/*
 * StringC.h - Simple string implementation.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 18-Jun-1998
 *
 */


#ifndef StringC_h
#define StringC_h 1

#include <typeinfo>
#include <fstream.h>
#include <iomanip.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>


/**
 * StringC implements simple string handling, like copying and concatenating. It has some extra features, such as casting to other types and input from and output to streams.
 *
 * @memo Simple string handling
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class StringC 
{
	public:




		/**
		 * @name Constructors.
		 *
		 * \begin{description}
		 * \item[] Examples: 
		 * \item[]{\tt StringC s("Test");}
		 * \item[]{\tt StringC t(s);}
		 * \item[]{\tt StringC r(24);}
		 * \item[]{\tt StringC i;}
		 * \item[]{\tt StringC n = "Str";}
		 * \item[]{\tt StringC g = i;}
		 * \end{description}
		 *
		 * @memo Making new strings
		 *
		 * @param iChar Char* to be copied.
		 * @param iInt Integer to be copied.
		 * @param iString String to be copied.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		//@{

		/// Standard constructor.
		StringC();

		/// string(char*)
		StringC
		(
			char const * const iChar
		);

		/// string(int)
		StringC
		(
			int iInt
		);

		/// string(string)
		StringC
		(
			StringC const & iString
		);

		//@}


		virtual ~StringC();



		/**
		 * @name Copy operators
		 *
		 * \begin{description}
		 * \item[] Examples: 
		 * \item[]{\tt StringC s;}
		 * \item[]{\tt StringC t;}
		 * \item[]{\tt s = "a string";}
		 * \item[]{\tt t = s;}
		 * \end{description}
		 *
		 * @memo Copying strings and char*s
		 * 
		 * @param iChar Char* to be copied.
		 * @param iString String to be copied.
		 * @return New string.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		//@{

		/// string = char*
		StringC & operator=
		(
			char const * const iChar
		);

		/// string = string
		StringC & operator=
		(
			StringC const & iString
		);

		//@}


		/**
		 *
		 * @name Comparison operators
		 *
		 * \begin{description}
		 * \item[] Example: 
		 * \item[]{\tt StringC s("Yes");}
		 * \item[]{\tt if (s != "No");}
		 * \subitem{\tt cout << "Equal";}
		 * \end{description}
		 * 
		 * @memo Comparing strings
		 * 
		 * @param iChar A constant string like "this is a string".
		 * @param iString A string of this class.
		 * @return When strings are equal: {\tt true} for operator== and {\tt false} for operator!=. 
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		//@{

		/// string == string
		bool const operator==
		(
			StringC const & iString
		) const;

		/// string == char*
		bool const operator==
		(
			char const * const iChar
		) const;

		/// char* == string
		friend bool const operator==
		(
			char const * const iChar,
			StringC const & iString
		)
		{
			return (iString == iChar);
		}

		/// string != char*
		bool const operator!=
		(
			char const * const iChar
		) const;

		/// string != string
		bool const operator!=
		(
			StringC const & iString
		) const;

		/// char* != string
		friend bool const operator!=
		(
			char const * const iChar,
			StringC const & iString
		)
		{
			 return !(iString == iChar);
		}
		//@}


		/**
		 *
		 * @name Concatenation operators
		 *
		 * \begin{description}
		 * \item[] Example: 
		 * \item[]{\tt StringC s("str");}
		 * \item[]{\tt StringC t("ing");}
		 * \item[]{\tt s += s + t;}
		 * \end{description}
		 *
		 * @memo Concatenating strings and char*s
		 * 
		 * @param iChar Char* to be concatenated.
		 * @param iString String to be concatenated.
		 * @return String concatenated with input.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		//@{

		/// char* + string
		friend StringC operator+
		(
			char const * const iChar,
			StringC const & iString
		)
		{
			StringC s(iChar);
			s += iString;
			return s;
		}

		/// string + char*
		StringC operator+
		(
			StringC const iString
		);

		/// string += char*
		StringC & operator+=
		(
			char const * const iChar
		);

		/// string += string
		StringC & operator+=
		(
			StringC const iString
		);
		//@}


		/**
		 * @name Stream operators
		 *
		 * \begin{description}
		 * \item[] Example: 
		 * \item[]{\tt StringC s("Something");}
		 * \item[]{\tt cout << s;}
		 * \end{description}
		 *
		 * @memo Getting and putting strings
		 * 
		 * @param iStream Stream to work on.
		 * @param iString String to be printed/read.
		 * @return Stream after handling string.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		//@{

		/// stream << string
		friend ostream & operator<<
		(
			ostream & iStream,
			const StringC & iString
		);

		/// stream >> string
		friend istream & operator>>
		(
			istream & iStream,
			StringC & iString
		);
		//@}


		/**
		 * @name Overload operators
		 *
		 * \begin{description}
		 * \item[] Example: 
		 * \item[]{\tt StringC s;}
		 * \item[]{\tt double d;}
		 * \item[]{\tt d = s;}
		 * \end{description}
		 *
		 * @memo Changing strings to other types
		 * 
		 * @return String as requested type.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		//@{

		/// double = string
		operator double const () const
		{
			return atof(String);
		};

		/// char* = string
		operator char const *() const
		{
			return (String);
		};
		//@}

	protected:

		/**
		 * Implementation that formats the output of the string that is printed
		 * to a stream.
		 *
		 * @param iStream Stream to output string on.
		 * @return Nothing.
		 *
		 * @see friend ostream & operator<<
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		virtual void const PrintToStream
		(
			ostream & iStream
		) const;

	private:

		/// Datatype for storing the actual string.
		char * String;

}; // StringC

#endif

// Eof StringC.h
