/*
 * ConfigurationA.h - A general configuration file reader with classes.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 18-Jun-1998
 *
 */

#ifndef ConfigurationA_h
#define ConfigurationA_h 1

#include "Common/StringC.h"
#include "Error/ErrorC.h"
#include "Error/WarningC.h"

#include <ostream.h>
#include <iostream.h>

/** The configuration class reads a file containing items looking like this: '{\tt NumberOfRuns = 10}'. It also makes use of classes, these look like: '{\tt [Experiment]}'. All items on lines below a line containing a class, and above a line containing some other class, belong to that first class. Using this items can be grouped together. For example:
 * \begin{description}
 * \item[]{\tt [Experiment]}
 * \item[]{\tt NumberOfRuns = 10}
 * \end{description}
 * Means that {\tt 'NumberOfRuns'} belongs to '{\tt Experiment}' and when we want to read this item in the program we just say:
 * \begin{description}
 * \item[]{\tt unsigned int NumberOfRuns}
 * \begin{description} \item[]{\tt = Configuration("Experiment", "NumberOfRuns");} \end{description}
 * \end{description}
 *
 * @memo Reading configuration files
 *
 * @author J.I. van Hemert
 * @version 0.1
 */

class ConfigurationA 
{
	public:

		virtual ~ConfigurationA () =0;

		/**
		 * @name Retrieval operators
		 *
		 * \begin{description}
		 * \item[] Example: 
		 * \item[]{\tt StringC s;}
		 * \item[]{\tt s = Configuration("class", "variable");}
		 * \item[]{\tt StringC t("runs");}
		 * \item[]{\tt s = Configuration("experiment", t);}
		 * \end{description}
		 *
		 * @memo Overload operators
		 * 
		 * @return String as requested type.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		//@{
		/// Configuration(string, string)
		virtual StringC const operator()
		(
			StringC const & iClass,
			StringC const & iItem
		) const =0;

		/// Configuration(char*, char*)
		virtual StringC const operator()
		(
			char const * iClass,
			char const * iItem
		) const =0;

		/// Configuration(char*, string)
		virtual StringC const operator()
		(
			char const * iClass,
			StringC const & iItem
		) const =0;

		/// Configuration(string, char*)
		virtual StringC const operator()
		(
			StringC const & iClass,
			char const * iItem
		) const =0;
		//@}

		friend ostream & operator<<
		(
			ostream & iStream,
			ConfigurationA const & iConfiguration
		)
		{
			iConfiguration.PrintToStream(iStream);

			return iStream;
		};

	protected:

		/// Prints the loaded configuration to the stream.
		virtual void const PrintToStream
		(
			ostream & iStream
		) const =0;

		/// This function retrieves an item given a class and a variable name.
		virtual StringC const GetStringItem
		(
			StringC const & iClass,
			StringC const & iItem
		) const throw(ExceptionA*) =0;

	private:

}; // ConfigurationA

#endif

// Eof ConfigurationA.h
