/*
 * CoreFactoryA.h - Abstract base class for instantiating objects within the Core package.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 18-Jun-1998
 *
 */

#ifndef CoreFactoryA_h
#define CoreFactoryA_h 1

#include "ConfigurationA.h"

class RandomA;
class ConfigurationA;
class ProblemA;
class ProblemFactoryA;
class ReproductionFactoryA;
class StatisticsA;

class EvolutionaryAlgorithmA;
class PopulationA;
class GenotypeA;
class InitializationA;
class StopConditionA;
class PoolA;


/** This factory is used to spawn new objects in the {\em Core} package of the library. When it is inherited the Create memberfunctions will decide which class to use for each different object that can be spawned. Thus by changing the factory all the objects in the ibrary will use other objects.
 *
 * @memo Creating objects within the {\em Core} package
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class CoreFactoryA
{
	public:

		/** This factory is inherited and the child determines for every Create member which class is used to spawn an object.
		 *
		 * @memo Creating a factory for the {\em Core} package
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iRandom A reference to this random object is stored and passed to classes who need it.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		CoreFactoryA
		(
			ConfigurationA const & iConfiguration,
			RandomA const & iRandom
		): Configuration(iConfiguration), Random(iRandom)
		{};


		virtual ~CoreFactoryA(){};


		/** 
		 * @name Create functions
		 *
		 * \begin{description}
		 * \item[] Examples: 
		 * \item[]{\tt GenotypeA * Genotype =}
		 * \begin{description} \item[]{\tt CoreFactory.CreateGenotype(ProblemFactory);} \end{description}
		 * \item[]{\tt PoolA * Pool =}
		 * \begin{description} \item[]{\tt CoreFactory.CreatePool();} \end{description}
		 * \end{description}
		 * Parameters depend on the needs of the constructor of the object that is being created. See the appropriate class for details.
		 *
		 * @memo Creating objects in the {\em Core} package
		 *
		 * @return An object of a class inherited from the appropriate abstract class.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		//@{
		/// Creating an EvolutionaryAlgorithm
		virtual EvolutionaryAlgorithmA* CreateEvolutionaryAlgorithm
		(
			StatisticsA & iStatistics,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			ProblemA const & iProblem
		) const =0;

		/// Creating a Population
		virtual PopulationA* CreatePopulation
		(
			StatisticsA & iStatistics,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			StopConditionA const & iStopCondition,
			ProblemA const & iProblem
		) const =0;

		/// Creating a Genotype
		virtual GenotypeA* CreateGenotype
		(
			ProblemFactoryA const & iProblemFactory
		) const =0;

		/// Creating a StopCondition
		virtual StopConditionA* CreateStopCondition() const = 0;

		/// Creating an Initialization
		virtual InitializationA* CreateInitialization
		(
			ProblemA const & iProblem
		) const =0;

		/// Creating a Pool
		virtual PoolA* CreatePool() const =0;
		//@}

	protected:

		ConfigurationA const & Configuration;

		RandomA const & Random;

	private:

}; // CoreFactoryA

#endif

// Eof CoreFactoryA.h
