/*
 * ProblemFactoryA.h - Abstract base class for instantiating objects within the Fitness package.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 18-Jun-1998
 *
 */

#ifndef ProblemFactoryA_h
#define ProblemFactoryA_h 1

#include "ConfigurationA.h"

class FitnessFunctionA;
class FitnessValueA;
class DataA;
class ProblemA;
class StopConditionA;
class StatisticsA;

/** This factory is used to spawn new objects in the {\em Problem} package of the library. When it is inherited the Create memberfunctions will decide which class to use for each different object that can be spawned. Thus by changing the factory all the objects in the ibrary will use other objects.
 *
 * @memo Creating objects within the {\em Problem} package
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class ProblemFactoryA
{
	public:

		/** This factory is inherited and the child determines for every Create member which class is used to spawn an object.
		 *
		 * @memo Creating a factory for the {\em Problem} package
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		ProblemFactoryA
		(
			ConfigurationA const & iConfiguration
		): Configuration(iConfiguration)
		{};


		virtual ~ProblemFactoryA()
		{};


		/** 
		 * @name Create functions
		 *
		 * \begin{description}
		 * \item[] Examples: 
		 * \item[]{\tt FitnessFunctionA * FitnessFunction =}
		 * \begin{description} \item[]{\tt ProblemFactory.CreateFitnessFunction(StopCondition, Statistics, Problem);} \end{description}
		 * \item[]{\tt DataA * Data =}
		 * \item[] \begin{description} \item[]{\tt ProblemFactory.CreateData(100);} \end{description}
		 * \end{description}
		 * Parameters depend on the needs of the constructor of the object that is being created. See the appropriate class for details.
		 *
		 * @memo Creating objects in the {\em Problem} package
		 *
		 * @return An object of a class inherited from the appropriate abstract class.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		//@{
		/// Creating a FitnessFunction
		virtual FitnessFunctionA* CreateFitnessFunction
		(
			StopConditionA const & iStopCondition,
			StatisticsA & iStatistics,
			ProblemA const & iProblem
		) const =0;

		/// Creating a Data object
		virtual DataA* CreateData
		(
			unsigned int iSize
		) const =0;

		/// Creating a Problem object
		virtual ProblemA* CreateProblem
		(
		) const =0;
		//@}

	protected:

		ConfigurationA const & Configuration;

	private:

}; // ProblemFactoryA

#endif

// Eof ProblemFactoryA.h
