/*
 * ProduceOperatorA.h - Abstract base class defining the interface of a produce operator.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Aug-1998
 *
 */

#ifndef ProduceOperatorA_h
#define ProduceOperatorA_h 1

#include "Common/ArrayC.h"
#include "Problem/FitnessFunctionA.h"
#include "Problem/DataA.h"
#include "Core/PoolA.h"
#include "Core/PoolIteratorA.h"
#include "Core/CoreFactoryA.h"
#include "Common/RandomA.h"

class RandomA;

/** This operator should produce new genotypes using the provided existing ones. The existing genotype are not changed. An other way of putting it: Given a number of parents this operator produces a number of children.
 *
 * @memo The production operator
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class ProduceOperatorA
{
	public:

		/** This constructor should only be called by its children. It copies the reference of the parameters into protected members, they can then be used by its children to spawn objects and read configuration parameters. 
		 *
		 * @memo Creating a new production operator
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iCoreFactory The factory that is used to spawn objects from classes of the Core package, within this class. It is passed on to these spawned objects as well.
		 * @param iProblemFactory The factory that is used to spawn objects from classes of the Problem package, within this class. It is passed on to these spawned objects as well.
		 * @param iRandom The production operator will require some random numbers.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		ProduceOperatorA
		(
			ConfigurationA const & iConfiguration,
			CoreFactoryA const & iCoreFactoryA,
			ProblemFactoryA const & iProblemFactory,
			ProblemA const &iProblem,
			RandomA const & iRandom
		);

		virtual ~ProduceOperatorA();


		/** \begin{description} \item[]{\tt ArrayC<GenotypeA const *>* Parents;} \item[]{\tt ArrayC<GenotypeA*>* Children;} \item[]{\em #/##/# Select parents and store them in the {\tt Parents} list} \item[]{\tt Children = ProduceOperator(*Parents);} \end{description} 
		 * This is the only function of the production operator. It returns a number of newly generated children using the provided list of existing ones.
		 *
		 * @memo Producing offspring
		 *
		 * @param iArray This is a list of parent genotypes.
		 * @return Newly generated genotypes.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual ArrayC<GenotypeA*>* operator()
		(
			ArrayC<GenotypeA const *> const & iArray
		) =0;
		
	protected:

		ConfigurationA const & Configuration;

		ProblemFactoryA const & ProblemFactory;
		CoreFactoryA const & CoreFactory;

		ProblemA const & Problem;

		RandomA const & Random;

	private:

}; // ProduceOperatorA

#endif

// Eof ProduceOperatorA.h
