/*
 * ReproductionFactoryA.h - Abstract base class for instantiating objects within the Fitness package.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Aug-1998
 *
 */

#ifndef ReproductionFactory_A
#define ReproductionFactory_A 1

#include "ConfigurationA.h"

#include "Core/CoreFactoryA.h"
#include "Problem/ProblemFactoryA.h"
#include "Common/RandomA.h"

class ProblemA;
class ReproductionA;
class SelectSurvivorsA;
class SelectParentsA;
class ChangeOperatorA;
class ProduceOperatorA;
class RandomA;
class StopConditionA;

/** This factory is used to spawn new objects in the {\em Reproduction} package of the library. When it is inherited the Create memberfunctions will decide which class to use for each different object that can be spawned. Thus by changing the factory all the objects in the ibrary will use other objects.
 *
 * @memo Creating objects within the {\em Reproduction} package
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class ReproductionFactoryA
{
	public:

		/** This factory is inherited and the child determines for every Create member which class is used to spawn an object.
		 *
		 * @memo Creating a factory for the {\em Reproduction} package
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iRandom A reference to this random object is stored and passed to classes who need it.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		ReproductionFactoryA
		(
			ConfigurationA const & iConfiguration,
			RandomA const & iRandom
		): Configuration(iConfiguration), Random(iRandom)
		{};


		virtual ~ReproductionFactoryA()
		{};


		/** 
		 * @name Create functions
		 *
		 * \begin{description}
		 * \item[] Examples:
		 * \item[]{\tt SelectParentsA * SelectParents =}
		 * \begin{description} \item[]{\tt ReproductionFactory.CreateSelectParents();} \end{description}
		 * \item[]{\tt ChangeOperatorA * ChangeOperator =}
		 * \begin{description} \item[]{\tt ReproductionFactory.CreateChangeOperator(ProblemFactory);} \end{description}
		 * \end{description}
		 * Parameters depend on the needs of the constructor of the object that is being created. See the appropriate class for details.
		 *
		 * @memo Creating objects in the {\em Reproduction} package
		 *
		 * @return An object of a class inherited from the appropriate abstract class.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		//@{
		/// Creating a Reproduction unit
		virtual ReproductionA* CreateReproduction
		(
			CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ProblemA const & iProblem,
			StopConditionA const & iStopCondition
		) const =0;

		/// Creating a Parents selections mechanism
		virtual SelectParentsA* CreateSelectParents() const =0;

		/// Creating a Surviviors selections mechanism
		virtual SelectSurvivorsA* CreateSelectSurvivors() const =0;

		/// Creating a Change operator
		virtual ChangeOperatorA* CreateChangeOperator
		(
			ProblemFactoryA const & iProblemFactory,
			ProblemA const & iProblem
		) const =0;

		/// Creating a Produce operator
		virtual ProduceOperatorA* CreateProduceOperator
		(
			CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ProblemA const & iProblem
		) const =0;
		//@}

	protected:

		ConfigurationA const & Configuration;
		RandomA const & Random;

	private:

}; // ReproductionFactoryA

#endif

// Eof ReproductionFactoryA.h
