/*
 * SelectParentsA.h - Abstract base class defining the interface of a parent selection mechanism.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 27-May-1998
 *
 */

#ifndef SelectParentsA_h
#define SelectParentsA_h 1

#include "Core/PoolA.h"
#include "Core/PoolIteratorA.h"
#include "Common/ArrayC.h"
#include "Common/RandomA.h"


/** This selection mechanism is used for filling a list of genotypes. These genotypes act as parents for the production of new offspring.
 *
 * @memo The parents selection mechanism
 *
 * @see ProduceOperator
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class SelectParentsA
{
	public:

		/** This constructor should only be called by its children. It copies the reference of the parameters into protected members, they can then be used by its children to spawn objects and read configuration parameters. 
		 *
		 * @memo Creating a new parent selection mechanism
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iRandom The mechanis will often require some random numbers.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		SelectParentsA
		(
			ConfigurationA const & iConfiguration,
			RandomA const & iRandom
		);


		virtual ~SelectParentsA();

		
		/** \begin{description} \item[]{\tt ArrayC<GenotypeA const *>* Parents;} \item[]{\tt Parents = SelectParents(Pool);} \end{description} 
		 * Given a pool filled with genotypes, this function selects a list of these genotypes and returns them.
		 *
		 * @memo Getting a list of parents
		 *
		 * @param iPool A pool with genotypes.
		 * @return An array filled with pointers to genotypes in the pool.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual ArrayC<GenotypeA const *>* operator()
		(
			PoolA const & iPool
		) const =0;
		
	protected:

		ConfigurationA const & Configuration;

		RandomA const & Random;

	private:

}; // SelectParentsA

#endif

// Eof SelectParentsA.h
